# -*- coding: utf-8 -*-
from time import time
import os
import logging
import hashlib
from datetime import datetime 
from zope.component.hooks import getSite

# --- sqlite3 --------
from config import DB_LOCATION, DB
# --- /sqlite3 --------

# --- psql ---------------
from config import PSQL_URI
# --- /psql ---------------

from config import DEBUG, USER_ID_BLACKLIST

# --- sqlalchemy -----
import sqlalchemy

from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, Integer, DateTime, Unicode, UnicodeText, Boolean
from sqlalchemy import desc, and_,  or_, not_, func
from sqlalchemy import ForeignKey
from sqlalchemy.orm import sessionmaker
from sqlalchemy.orm import relationship, backref

# fuer mysql/psql
from sqlalchemy import String, Text
from config import TABLE_MESSAGES, TABLE_MESSAGE_FLAGS, TABLE_MESSAGE_GROUPS

Base = declarative_base()


class Message(Base):
    """ An message Object
    """
    __tablename__ = TABLE_MESSAGES

    # --- psql --------------------------    
    id              = Column(Integer, primary_key=True)
    md5             = Column(String, unique=True)
    sender_id       = Column(Unicode)
    sender_name     = Column(Unicode)
    sender_email    = Column(Unicode)
    receiver_id     = Column(Unicode)
    receiver_name   = Column(Unicode)
    receiver_email  = Column(Unicode)
    message         = Column(UnicodeText)
    timestamp       = Column(DateTime)
    flags   = relationship("MessageFlag")
    # --- /psql --------------------------
   
class MessageGroup(Base):
    """ An message group Object
    """
    __tablename__ = TABLE_MESSAGE_GROUPS
    
    # --- psql --------------------------
    id              = Column(Integer, primary_key=True)
    group_hash      = Column(Unicode)
    group_name      = Column(Unicode)
    receiver_id     = Column(Unicode)
    # --- /psql --------------------------

class MessageFlag(Base):
    """ An message flag Object
    """
    __tablename__ = TABLE_MESSAGE_FLAGS
    
    # --- psql --------------------------
    id              = Column(Integer, primary_key=True)
    message_md5     = Column(String, ForeignKey(  TABLE_MESSAGES + '.md5' ))
    user_id         = Column(Unicode)
    hide            = Column(Boolean)
    approved        = Column(Boolean)
    deleted         = Column(Boolean)
    message = relationship("Message", backref=backref(TABLE_MESSAGE_FLAGS, order_by=id))
    # --- /psql --------------------------

class DBApi(object):
    """ DB Api to sqlite3
    """
    
    def __init__(self):
        """Initialize Database
        """

        ## --- psql ----------------------
        engine  = create_engine(PSQL_URI,  client_encoding='utf8', echo=False)
        ## --- /psql ----------------------

        self.Session = sessionmaker(bind=engine)
        Base.metadata.create_all(engine)
        self.db_utils = DBUtils()


    def countNewMessages(self, receiver_id=None, sender_id=None, \
                    timestamp=None, approved=True):
        """ count messages
        """
        count = 0
        try:
            se = self.Session()
            q = se.query(Message.id).filter(Message.hide == None)
            if receiver_id:
                q = q.filter(Message.receiver_id == receiver_id)
                
            if sender_id:
                q = q.filter(Message.sender_id == sender_id)
                
            if timestamp:
                q = q.filter(Message.timestamp > timestamp)
                    
            count = q.count()

        except:
            logging.error('Error while executing countNewMessages')
        finally:            
            se.close()

        return count


    def getMessages(self, receiver_ids=None, sender_id=None, \
                    timestamp=None, newer=False, group_field=None, \
                    order_field="id", reverse_order=True, \
                    approved=False, omit_sender=False, max=20,offset=0, auth_user_id=None, dialog=False):
        """ get the messages of a givern receiver
        of if no receiver is passed: return all
        messages

        """

        #print "----------- getMessages params --------------------"
        #print "receiver_ids   ", receiver_ids
        #print "sender_id      ", sender_id
        #print "timestamp      ", timestamp
        #print "newer          ", newer
        #print "group_field    ", group_field
        #print "order_field    ", order_field
        #print "approved       ", approved
        #print "omit_sender    ", omit_sender
        #print "max            ", max
        #print "offset         ", offset
        #print "auth_user_id   ", auth_user_id
        #print "---------------------------------------------------"

        t1 = time()
        messages = None
        res = []

        if max > 50: max = 50 # never > 50 !

        try:        
            se = self.Session()

            if group_field:
                q = se.query( func.count(Message.id), Message, MessageFlag).outerjoin( MessageFlag, and_(MessageFlag.message_md5 == Message.md5, MessageFlag.user_id == auth_user_id) )
                """select * from messages ....
                """
            else:
                q = se.query(Message, MessageFlag).outerjoin(MessageFlag, and_(MessageFlag.message_md5 == Message.md5, MessageFlag.user_id == auth_user_id))
            
            if approved == "true":
                q = q.filter(or_(MessageFlag.approved == True, MessageFlag.approved == None, MessageFlag.user_id == None))
            elif approved == "all":
                pass
            else:
                q = q.filter(or_(MessageFlag.approved == False, MessageFlag.approved == None, MessageFlag.user_id == None))

            if receiver_ids and dialog:
                q = q.filter(or_(MessageFlag.hide == False, MessageFlag.hide == None, MessageFlag.user_id == None), or_(Message.receiver_id.in_(receiver_ids), Message.sender_id.in_(receiver_ids)))

            if receiver_ids and not dialog:
                q = q.filter(or_(MessageFlag.hide == False, MessageFlag.hide == None, MessageFlag.user_id == None), Message.receiver_id.in_(receiver_ids))

            if sender_id:
                q = q.filter(or_(MessageFlag.hide == False, MessageFlag.hide == None, MessageFlag.user_id == None), Message.sender_id == sender_id)

            if timestamp:
                if newer:
                    q = q.filter(Message.timestamp > timestamp)
                else:
                    q = q.filter(Message.timestamp < timestamp)
                    
            if omit_sender:
                q = q.filter( Message.sender_id != omit_sender)

            if order_field:
                if reverse_order:
                    q = q.order_by(desc(getattr(Message,order_field)))
                else:
                    q = q.order_by(getattr(Message,order_field))

            if group_field:
                q = q.group_by(group_field, Message.id, MessageFlag.id)

            messages = q[offset:offset+max]
            
            if group_field:
                res =  [  (message[0], self.db_utils.convertAttributesToDict(message[1], message[2]) ) for message in messages  ]
                print "\t\tDAUER getMessages (group_field) (ms):", (time()-t1)*1000            
            else:      
                res = [  self.db_utils.convertAttributesToDict(message[0], message[1])  for message in messages  ]
                print "\t\tDAUER getMessages (ms):", (time()-t1)*1000

        except:
            logging.error('Error while executing getMessages')
        finally:            
            se.close()

        return res


    def getMessage(self, md5):
        """ get amessage with a given md5 key
        """        
        message = None
        t1 = time()
        try:
            se = self.Session()
            message = se.query(Message).filter(Message.md5 == md5).one()        
        except:
            logging.error("Retrieving message %s not possible" % md5)            
        finally:
            se.close()
        
        if message:
            res = self.db_utils.convertAttributesToDict(message, None)
            print "\t\tDAUER getMessage (ms):", (time() - t1)*1000
            return res
        else:
            return False
        

    def addMessage(self,nm):
        """ Add a message to the database
            nm: New Message
        """
        t1 = time()
               
        md5 = hashlib.new('md5', str(nm) + str(datetime.now())).hexdigest()            
        timestamp = datetime.now()
         
        m = Message(
            md5             = md5,
            sender_id       = nm['sender_id'],
            sender_name     = nm['sender_name'],
            sender_email    = nm['sender_email'],
            receiver_id     = nm['receiver_id'],
            receiver_name   = nm['receiver_name'],
            receiver_email  = nm['receiver_email'],
            message         = nm['message'],
            timestamp       = timestamp
            )
        try:
            se = self.Session()
            se.add(m)
            se.commit()
        except:
            logging.error("Could not insert message %s" % str(m))
            return False
        finally:
            se.close()
            
        print "\t\tDAUER addMessage (ms):", (time()-t1)*1000
        return { 'md5':md5, 'timestamp': timestamp }


    def setMessageState(self,receiver_id,mids,state):
        """ Set the state of a message: delete or read
            possible states:
                - approved: message is approved by receiver
                - deleted: message is deleted by receiver
                - hide: hided the message
        """
        #timestamp = datetime.now()
        t1 = time()

        for md5 in mids:
            try:
                se = self.Session()
                mf = se.query(MessageFlag)
                mf = mf.filter(MessageFlag.message_md5 == md5)
                mf = mf.filter(MessageFlag.user_id == receiver_id)
                if len(mf.all()) == 0:
                    new_mf = MessageFlag(message_md5 = md5,
                                          user_id = receiver_id,
                                          hide = False,
                                          approved = False,
                                          deleted = False)
                    se.add(new_mf)
                    se.commit()
                    se.close()
                    se = self.Session()
                    mf = se.query(MessageFlag).filter(MessageFlag.message_md5 == md5)
                    mf = mf.filter(MessageFlag.user_id == receiver_id).one()
                else:
                    mf = mf.one()
                if state == "approved":
                    mf.approved = True
                elif state == "deleted":
                    mf.deleted = True
                elif state == "hide_receiver":
                    mf.hide = True
                elif state == "hide_sender":
                    mf.hide = True
                else:
                    logging.warning("setMessageState: '%s' is not a valid state" % state)
                    
                #m.timestamp = timestamp # Warum???
                se.commit()         
            except:
                logging.error("Could not '%s' message" % state)
            finally:
                se.close()        

        print "\t\tDAUER getMessageStatus (ms):", (time()-t1)*1000
        return 
        
    def updateMessage(self, md5, nm, receiver_id):
        """ Updates a message wit a given md5 in the database
            nm: New Message, 
            returns the updated text-message
        """
        message = None
        timestamp = datetime.now()
            
        try:
            se = self.Session()
            m = se.query(Message).filter(Message.md5 == md5).one()
            mfs = se.query(MessageFlag).filter(MessageFlag.message_md5 == md5, MessageFlag.user_id == receiver_id).all()
            for mf in mfs:
                mf.hide = False
                mf.approved = False
                mf.deleted = False
            message = m.message + '<br />' + nm
            m.message = message
            m.timestamp = timestamp
            se.commit()
        except:
            logging.error("Could not update Comment" )
        finally:
            se.close()

        return { 'md5':md5, 'timestamp': timestamp }
    

    def getLatestMessageOfAllSenders(self, receiver_id=None, timestamp=None, order_field="id", reverse_order=True, max=20,offset=0):
        """Returns all senders newest message 
        """
        t1 = time()
        if not receiver_id:
            return None
        
        messages = []

        try:
            se = self.Session()            
            q = se.query(Message)

            if receiver_id:
                q = q.filter(Message.receiver_id == receiver_id)
                         
            if order_field:
                if reverse_order:
                    q = q.order_by(desc(getattr(Message,order_field)))
                else:
                    q = q.order_by(getattr(Message,order_field))
                    
            if timestamp:
                if reverse_order:
                    q = q.filter(Message.timestamp > timestamp)
                else:
                    q = q.filter(Message.timestamp < timestamp)
                    
            q = q.group_by(Message.sender_id)
                    
            messages = q[offset:offset+max]
                    
        except:
            logging.error('Error while executing getLatestMessageOfAllSenders')
        finally:
            se.close()

        print "\t\tDAUER getLatestMessageOfAllSenders (ms):", (time()-t1)*1000

        if messages:
            return [  self.db_utils.convertAttributesToDict(message) for message in messages  ]
        
    
    def getDialog(self, sender_id, receiver_id, \
                        timestamp=None, newer=False, \
                        order_field="", reverse_order=False, \
                        max=20, offset=0, approved=False, auth_user_id = False):
        """ Returns the dialog between 2 members
        """
        t1 = time()
        messages = None

        try:
            se = self.Session()
            
            q = se.query(Message, MessageFlag).outerjoin(MessageFlag, and_(MessageFlag.message_md5 == Message.md5, MessageFlag.user_id == auth_user_id))
            q = q.filter(or_(and_(Message.sender_id == sender_id,Message.receiver_id == receiver_id),\
                             and_(Message.sender_id == receiver_id,Message.receiver_id == sender_id)))

            q = q.filter(or_(MessageFlag.hide == None, MessageFlag.hide == False))
            if approved == "true":
                q = q.filter(MessageFlag.approved == True)
            elif approved == "all":
                pass
            else:
                q = q.filter(or_(MessageFlag.approved == False, MessageFlag.approved == None))
            
            if timestamp:
                if newer:
                    q = q.filter(Message.timestamp > timestamp)
                else:
                    q = q.filter(Message.timestamp < timestamp)
                    
            if order_field:
                if reverse_order:
                    q = q.order_by(desc(getattr(Message,order_field)))                
                else:
                    q = q.order_by(getattr(Message,order_field))

            messages = q[offset:offset+max]
            messages.reverse()

        except:
            logging.error('Error while executing getLatestMessageOfAllSenders')        
        finally:
            se.close()

        print "\t\tDAUER getDialog (ms):", (time()-t1)*1000
        if messages:
            return [  self.db_utils.convertAttributesToDict(message[0], message[1]) for message in messages  ]

    
    def createGroup(self, receiver_ids, group_name):
        """ Creates a messaging group in database
        """
        m = hashlib.md5()
        m.update(str(receiver_ids))
        m.update(str(datetime.now()))
        hash = m.hexdigest()
        group_entries = []
        for id in receiver_ids:
            group_entries.append(MessageGroup(
                                             group_hash = m.hexdigest(),
                                             group_name = group_name,
                                             receiver_id = id
                                             ))
        try:
            se = self.Session()
            se.add_all(group_entries)
            se.commit()

        except:
            logging.error('Error while executing createGroup')        
        finally:
            se.close()
        
        return hash

    
    def getReceiversOfGroup(self, md5):
        """ Get receivers of group by hash
        """
        receiver_list = None
        try:
            se = self.Session()
            receivers = se.query(MessageGroup.receiver_id).filter(MessageGroup.group_hash == md5).all()
            receiver_list = []
            if receivers:
                for r in receivers:
                    receiver_list.append(r[0])
        except:
            logging.error('Error while executing getReceiversOfGroup')
        finally:
            se.close()

        return receiver_list

    
    def getGroupname(self, md5):
        """ Get group name for md5
        """
        res = None
        try:
            se = self.Session()
            groupname = se.query(MessageGroup.group_name).filter(MessageGroup.group_hash == md5).first()
            res = groupname[0]
        except:
            logging.error('Error while executing getGroupname')
        finally:
            se.close()

        return res

    
    def getGroupsOfUser(self, user_and_groups):
        """ Gets all imessage-groups of a user.
        Given is a list with the user_id and his groups
        """
        hash_list = []
        try:
            se = self.Session()
            hashs = se.query(MessageGroup.group_hash).filter(MessageGroup.receiver_id.in_(user_and_groups)).all()
            for hash in hashs:
                hash_list.append(hash[0])
        except:
            logging.error('Error while executing getGroupsOfUser')
        finally:
            se.close()

        return hash_list
    
    def deleteMessagesOfUser(self, uid):
        """ Deletes all messages of an user
        """
        se = self.Session()
        try:
            q = se.query(Message.md5).filter(or_(Message.receiver_id == uid, Message.sender_id == uid)).all()
            message_md5s = []
            for md5 in q:
                if not md5[0] in message_md5s:
                    message_md5s.append(md5[0])
            
            se.query(MessageFlag).filter(MessageFlag.message_md5.in_(message_md5s)).delete(synchronize_session='fetch')
            se.commit()
            se.query(Message).filter(or_(Message.receiver_id == uid, Message.sender_id == uid)).delete(synchronize_session='fetch')
            se.commit()
        except:
            logging.error("Couldn't delete messages for user %s" % uid)
        finally:
            se.close()
        
        return True

class DBUtils():
    """ Help functions / Utils
    """
    def convertAttributesToDict(self, object, object_flags):
        """Returns all public attributes of an object as a dict of attributes"""
        adict = dict((key, value) for key, value in object.__dict__.iteritems() 
                   if not callable(value) and not key.startswith('_'))
        if object_flags == None:
            adict["approved"] = False
            adict["deleted"] = False
        else:
            flagdict = dict((key, value) for key, value in object_flags.__dict__.iteritems() 
                   if not callable(value) and not key.startswith('_'))
            adict.update(flagdict)
        
        return adict



# -*- coding: utf-8 -*-
from plone.memoize.instance import memoize
from Products.CMFCore.utils import getToolByName 
from Acquisition import aq_inner
from zope.component.hooks import getSite
try:
    from BeautifulSoup import BeautifulSoup as bs
except:
    from bs4 import BeautifulSoup as bs
    
from ..config import TIME_STRING

from .. import isProductAvailable

class InstantMessageUtils:
    
    def senders(self,context,show_all):
        """Placeholder, TODO
        """
        senders = []
        return senders
    

    def convertMessagesForTemplate(self, context, ms):
        """ Converts the message dict to what the template needs
            ms may be a list of message-obj or
            a list of tuple like (count,<message>)
        """
        messages = []
        if not ms:
            return messages
        mt       = getToolByName(context,'portal_membership')
        
        for _m in ms:
            if type(_m) == type((0,0)):
                c, m = _m[0], _m[1]
            else:
                c, m = 1, _m
                                
            sender_id         = m['sender_id']
            sender_name       = m['sender_name']
            receiver_id       = m['receiver_id']
            receiver_name     = m['receiver_name']
            
            sender_portrait   = self.getPersonalPortrait(sender_id, size='icon').absolute_url()
            receiver_portrait = self.getPersonalPortrait(receiver_id, size='icon').absolute_url()
            timestr           = m['timestamp'].strftime(TIME_STRING)
            
            m_tmpl = {
                'sender_id':        sender_id,
                'sender_name':      sender_name,
                'sender_email':     m['sender_email'],
                'sender_portrait':  sender_portrait,
                'receiver_id':      m['receiver_id'],
                'receiver_name':    m['receiver_name'],
                'receiver_email':   m['receiver_email'],
                'receiver_portrait': receiver_portrait,
                'message':          m['message'],
                'timestamp':        str(m['timestamp']),
                'timestr':          timestr,    
                'hash':             m['md5'],
                'count':            c,
                'approved':         m['approved']
            }
            messages.append(m_tmpl)
                        
        return messages

    @memoize    
    def getSmallPortrait(self,portrait,username,size="small-personal-portrait"):
        """ rermoves width and height attributes
            and adds a class to the img tag
            if jquery is rendering the image, we only need
            the src of the image
            TO BE DELETED
        """
        
        soup = bs(portrait)
        img = soup.img

        return str(img['src'])

    @memoize
    def getPersonalPortrait(self,id=None, verifyPermission=0, size=None):
        """Adapts the original getPersonalPortrait 
        If ityou.extpersonalportrait is installed, the
        patched personal portrai will be called, else the
        default
        """
        plone = getSite()
        mt = getToolByName(plone,"portal_membership")
        if isProductAvailable('ityou.extuserprofile'):
            return mt.getPersonalPortrait(id, size=size)
        else:
            return mt.getPersonalPortrait(id=id)




# -*- coding: utf-8 -*-
import logging
from zope.component.hooks import getSite
from zope.i18nmessageid import MessageFactory
from zope.component import getUtility
from plone.registry.interfaces import IRegistry
_ = MessageFactory('ityou.imessage')


def isProductAvailable(product):
    """Check if a product is installed and return True, 
    else return False
    """
    qui = getSite().portal_url.getPortalObject().portal_quickinstaller
    installed_products = qui.listInstalledProducts()
    for prod in installed_products:
        if prod["id"] == product and prod["status"] == "installed":
            return True
    return False

def getNotifyDBApi():
    """Loads Notify DBApi Function
    Returns False if the product is not installed
    """
    dbapi_notify = False
    
    qui = getSite().portal_url.getPortalObject().portal_quickinstaller
    installed_products = qui.listInstalledProducts()
    
    for product in installed_products:
        if product["id"] == "ityou.notify" and product["status"] == "installed":
            try:
                from ityou.notify.dbapi import DBApi
                from ityou.notify.interfaces import INotifySettings
                if getUtility(IRegistry).forInterface(INotifySettings).imessage_notifications_active:
                    dbapi_notify = DBApi()
                else:
                    logging.info("ityou.imessage says: ITYOU Notify App is deactivated - No emails will be send")
            except:
                logging.info("ityou.imessage says: Could not load ITYOU Notify App - No emails will be send")
            
    return dbapi_notify

def initialize(context):
    """Initializer called when used as a Zope 2 product."""



# -*- coding: utf-8 -*-
"""
import random
from Acquisition import aq_inner

from zope import schema
from zope.formlib import form
from zope.interface import implements
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component.hooks import getSite

from plone.registry.interfaces import IRegistry
from plone.app.portlets.portlets import base
from plone.memoize.instance import memoize
from plone.portlets.interfaces import IPortletDataProvider
from DateTime import DateTime
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName

from ..config import MIN_IMESSAGE_DELAY
from ..interfaces import IInstantMessageSettings
from .. import _

from .utils import InstantMessageUtils

class IInstantMessagePortlet(IPortletDataProvider):

    count = schema.Int(
            title=_(u"Number of users to display"),
            description=_(u"Maximum number of users to show"),
            required=True,
            default=10,
        )
                       
class Assignment(base.Assignment):
    implements(IInstantMessagePortlet)

    def __init__(self, count=20):
        self.count = count

    title = _(u"Instant Messages")

class Renderer(base.Renderer):

    _template = ViewPageTemplateFile('imessage.pt')
    
    def render(self):

        context = aq_inner(self.context)
        mt = getToolByName(context, 'portal_membership')
        
        ru =  getUtility(IRegistry)
        # *1000: transform milliseconds to seconds
        # We put this value in the Portlet DOM so that
        # jquery can fetch it
        # if value lower than MIN_IMESSAGE_DELAY,
        # we take MIN_IMESSAGE_DELAY
        self.IMESSAGE_DELAY = max([ru.forInterface(IInstantMessageSettings).imessage_delay*1000, MIN_IMESSAGE_DELAY])
        self.ESI_ROOT = getSite().absolute_url()
        # only show if not Anonymous    
        if not mt.isAnonymousUser():
            return self._template()
              
    @property
    def available(self):
        return True

    def senders(self):
        "Show users who send messages   #TODO "
        # extern utils; no need to restart zope on changes        
        who = InstantMessageUtils()
        limit = self.data.count
        return who.senders()[:limit]

class AddForm(base.AddForm):
    form_fields = form.Fields(IInstantMessagePortlet)
    label = _(u"Add IMessage portlet")
    description = _(u"This portlet displays users tat send messages.")

    def create(self, data):
        assignment = Assignment()
        form.applyChanges(assignment, self.form_fields, data)
        return assignment

class EditForm(base.EditForm):
    form_fields = form.Fields(IInstantMessagePortlet)
    label = _(u"Edit IMessage portlet")
    description = _(u"This portlet displays online users.")
"""    



#-*- coding: utf-8 -*-
from plone.app.registry.browser import controlpanel
from ityou.imessage.interfaces import IInstantMessageSettings, _

class InstantMessageSettingsEditForm(controlpanel.RegistryEditForm):

    schema = IInstantMessageSettings
    label = _(u"Instant message settings")
    description = _(u"""Settings of the 'imessage' variables
    """)

    def updateFields(self):
        super(InstantMessageSettingsEditForm, self).updateFields()

    def updateWidgets(self):
        super(InstantMessageSettingsEditForm, self).updateWidgets()

class InstantMessageSettingsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = InstantMessageSettingsEditForm


# -*- coding: utf-8 -*-
import logging
import time
import json
import hashlib
from datetime import datetime, date, timedelta

from Acquisition import aq_inner

from zope.component.hooks import getSite
from zope.interface import implements
from zope.component import getUtility
from zope.component import getMultiAdapter

from plone.memoize.instance import memoize
from plone.registry.interfaces import IRegistry
from plone.portlets.interfaces import IPortletDataProvider

from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName

from ..dbapi import DBApi
from ..config import TIME_STRING
from ..config import MIN_IMESSAGE_DELAY
from ..interfaces import IInstantMessageSettings
from .. import isProductAvailable

from plone.outputfilters.browser.resolveuid import uuidToObject, uuidFor

from stripogram import html2text
try:
    from BeautifulSoup import BeautifulSoup as bs
except:
    from bs4 import BeautifulSoup as bs


from zope.i18n import translate


class DialogOverview(BrowserView):
    """View of an Instant Message
    """

    html_template = ViewPageTemplateFile('templates/dialog-overview.pt')

    def __call__(self):
        self.request.set('disable_border', True)        
        
        self.portal = getSite()
        self.portal_url = getSite().absolute_url()

        ru =  getUtility(IRegistry)
        # *1000: transform milliseconds to seconds
        # We put this value in the Portlet DOM so that
        # jquery can fetch it
        # if value lower than MIN_IMESSAGE_DELAY,
        # we take MIN_IMESSAGE_DELAY
        self.IMESSAGE_DELAY = max([ru.forInterface(IInstantMessageSettings).imessage_delay*1000, MIN_IMESSAGE_DELAY])
        self.ESI_ROOT = self.portal_url
        
        return self.html_template()





# -*- coding: utf-8 -*-
from datetime import datetime, timedelta 
import logging

from dbapi import DBApi


class DBTest():
    """Tests for the DB
    """
    
    def __init__(self):    
        self.db = DBApi()
        self.tdelta = datetime.now() - timedelta(seconds = 3600)

        self.md5 = self.test_addMessage()['md5']
        self.test_getMessage(self.md5)['md5']
        self.test_getMessages(timestamp=self.tdelta)
        self.test_updateMessage(self.md5, u'Das ist ein Update! ÖÄÜ')
        
        self.test_getDialog(u'lmuller', u'ppan')

    def test_addMessage(self):        
        message1 = {
            'sender_id':        u'lmuller',
            'sender_name':      u'Luc Müller',
            'sender_email':     u'lmuller@ityou.de',
            'receiver_id':      u'ppan',
            'receiver_name':    u'Petra Pan',
            'receiver_email':   u'support@ityou.de',
            'message':          u'Das ist der Kern der Mitteilung:ÖÄÜß',
            'approved':         False
        }
        message2 = {
            'sender_id':        u'ppan',
            'sender_name':      u'Peter Pan',
            'sender_email':     u'support@ityou.de',
            'receiver_id':      u'lmuller',
            'receiver_name':    u'Luc Müller',
            'receiver_email':   u'lmuller@ityou.de',
            'message':          u'Das ist eine Mitteilung von pp an lm',
            'approved':         False
        }
        message3 = {
            'sender_id':        u'hmustermann',
            'sender_name':      u'Hans Mustermann',
            'sender_email':     u'support@ityou.de',
            'receiver_id':      u'lmuller',
            'receiver_name':    u'Luc Müller',
            'receiver_email':   u'lmuller@ityou.de',
            'message':          u'Das ist eine Mitteilung von pp an lm',
            'approved':         False
        }
        message3 = {
            'sender_id':        u'hmustermann',
            'sender_name':      u'Hans Mustermann',
            'sender_email':     u'support@ityou.de',
            'receiver_id':      u'ppan',
            'receiver_name':    u'Petra Pan',
            'receiver_email':   u'info@ityou.de',
            'message':          u'Das ist eine Mitteilung von hh an pp',
            'approved':         False
        }

        db_message = self.db.addMessage(message3)
        
        logging.info("\n\nADD MESSAGE:\n\t\t %s\n" % str(db_message))
        return db_message
        
        
    def test_getMessage(self, md5):
        db_message = self.db.getMessage(md5=md5)
        logging.info("\n\n GET MESSAGE: \n\t\t %s \n" % str( db_message ))
        return db_message
        

    def test_getMessages(self, timestamp):
        db_messages =  [ str(m) for m in self.db.getMessages(timestamp=timestamp) ]  
        logging.info("\n\n GET MESSAGES: \n\t\t %s \n" % str(  db_messages   ))
        return db_messages
        
    def test_updateMessage(self, md5, message):
        db_message = self.db.updateMessage(md5, message)
        logging.info("\n\n UPDATE MESSAGE: \n\t\t: %s \n" %  db_message)
        return db_message

    def test_getDialog(self, sender_id, receiver_id):
        db_messages = self.db.getDialog(sender_id=sender_id, receiver_id=receiver_id)
        logging.info(str("\n\n DIALOG \n\t\t: %s\n" %  db_messages  ))
        return db_messages



#-*- coding: utf-8 -*-
from z3c.form import interfaces

from zope import schema
from zope.interface import Interface

from zope.i18nmessageid import MessageFactory
_ = MessageFactory('ityou.astream')

class IInstantMessageSettings(Interface):
    """Global instant message settings. This describes records stored in the
    configuration registry and obtainable via plone.registry.
    """
    imessage_delay = schema.Int(
            title=_(u"Time period for ajax requests"),
            description=_(u"Enter the time period between to Ajax-Requests. \
                    IMPORTANT: Be aware that time period less then 10 second may slow down your server! \
                    Time periods less then 4 seconds are not considered."),
            required=True,
            default=10,
        )


class IInstantMessage(Interface):
    """Marker Interface
    """




# -*- coding: utf-8 -*-
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.Five.browser import BrowserView

from Products.CMFCore.utils import getToolByName 
from Acquisition import aq_inner
from zope.component.hooks import getSite

from datetime import datetime, timedelta, date
import logging

from operator import itemgetter
from plone.memoize.instance import memoize

from ..dbapi import DBApi
from utils import InstantMessageUtils

from ..config import TIMEOUT

db  = DBApi()
mu = InstantMessageUtils()

class UserProfile():
    """Define the user profile
    """
    id          = u""                              
    fullname    = u"" 
    email       = u"" 
    profile     = u""
    home        = u""
    portrait    = u""
    recent_path = u""
    recent_doc  = u""
    recent_time = u""
    bar         = u""

    
class AjaxInstantMessagePortletView(BrowserView):
    """AJAX list of online users 
    """
    
    template = ViewPageTemplateFile('imessage-ajax.pt')
    
    def __call__(self):
        context = aq_inner(self.context)
        request = context.REQUEST
        request.set('disable_border', True)
        return self.template()

    def get_latest_senders(self):
        """ Returns the latest message of all senders
        slm: senders_latest_message
        """        
        slm = []
        context = aq_inner(self.context)
        mt	    = getToolByName(context,'portal_membership')
        user_id	= mt.getAuthenticatedMember().getId()
        
        if user_id:
            slm = db.getMessages(receiver_ids=[user_id], group_field="sender_id", omit_sender=user_id, auth_user_id = user_id)
            slm = mu.convertMessagesForTemplate(context, slm)
            return slm
        else:
            return None
        
    def get_sender_latest_messages(self,sender_id):
        """ Returns the latest message and the amount of 
        open messages of all senders
        slm: senders_latest_message
        """
        slm = []
        
        context = aq_inner(self.context)
        mt        = getToolByName(context,'portal_membership')
        user_id    = mt.getAuthenticatedMember().getId()
        
        if user_id:
            #slm = db.getMessages(receiver_ids=[user_id],sender_id=sender_id) #BUG ############### auth_user_id fehlt!
            slm = db.getMessages(receiver_ids=[user_id],sender_id=sender_id, auth_user_id = user_id) 
            slm = mu.convertMessagesForTemplate(context, slm)
                        
            return slm
        else:
            return None
        
    def clip(self, message, count):
        """ Clip string, count words
        """
        ml = message.split(' ')
        
        if len(ml) > count:
            message = " ".join(ml[:count]) + " (...)"
        else:
            message = " ".join(ml)
        
        return message



from setuptools import setup, find_packages
import os

version = '1.3rc1'

setup(name='ityou.imessage',
      version=version,
      description="Instant Messaging for ITYOU ESI",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        ],
      keywords='Messaging',
      author='ITYOU/LM',
      author_email='lm@ityou.de',
      hidden=True,
      url='http://svn.plone.org/svn/collective/',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ityou'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'ityou.esi.theme',
          'sqlalchemy',
          'stripogram',
          'BeautifulSoup',
          'psycopg2'
      ],
      entry_points="""
      # -*- Entry points: -*-

      [z3c.autoinclude.plugin]
      target = plone
      """,
      setup_requires=["PasteScript"],
      paster_plugins=["ZopeSkel"],
      )




# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from Products.CMFCore.utils import getToolByName 
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.app.layout.viewlets.common import ViewletBase    


from .dbapi import DBApi
db  = DBApi()

class MessageTrackingViewlet(ViewletBase):
    """Checks if user got an new message
    """
    
    def update(self):
        super(MessageTrackingViewlet, self).update()
                  
        context = aq_inner(self.context)        
        mt = getToolByName(context,"portal_membership")
        user = mt.getAuthenticatedMember()
         
        if user.getId():
            pass


# -*- coding: utf-8 -*-
import logging
from time import time
import json
import hashlib
import re
from datetime import datetime, date, timedelta

from Acquisition import aq_inner

from zope.component.hooks import getSite
from zope.interface import implements
from zope.component import getUtility
from zope.component import getMultiAdapter

from plone import api

from plone.memoize.instance import memoize
from plone.registry.interfaces import IRegistry
from plone.portlets.interfaces import IPortletDataProvider

from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName

from ..dbapi import DBApi
from ..config import TIME_STRING
from ..config import MIN_IMESSAGE_DELAY
from ..interfaces import IInstantMessageSettings
from .. import isProductAvailable
from .. import getNotifyDBApi

from plone.outputfilters.browser.resolveuid import uuidToObject, uuidFor

from stripogram import html2text
try:
    from BeautifulSoup import BeautifulSoup as bs
except:
    from bs4 import BeautifulSoup as bs

from plone import api
from zope.i18n import translate

try:
    from ityou.whoisonline.dbapi import RDBApi as WhoRDBApi
    who_rdb = WhoRDBApi()
except:
    who_rdb = False

# extended user profile
from ityou.extuserprofile.dbapi import DBApi as DBApi_extuserprofile
db_extuserprofile = DBApi_extuserprofile()

# imessage dbapi
db = DBApi()  

# notification via email
from ityou.notify.dbapi import DBApi as NotifyDBApi
from ityou.notify.interfaces import INotifySettings


# ------ redis ---------------------------------------
from ityou.esi.theme.dbapi import RDBApi
rdb = RDBApi()
# ------ /redis --------------------------------------

from ..config import IMESSAGE_REGEX

class InstantMessageView(BrowserView):
    """View of an Instant Message
    """

    html_template = ViewPageTemplateFile('templates/imessage.pt')

    def __call__(self):
        self.request.set('disable_border', True)        
        self.mu = MessageUtils()
        
        self.portal = getSite()
        self.portal_url = getSite().absolute_url()

        ru =  getUtility(IRegistry)
        # *1000: transform milliseconds to seconds
        # We put this value in the Portlet DOM so that
        # jquery can fetch it
        # if value lower than MIN_IMESSAGE_DELAY,
        # we take MIN_IMESSAGE_DELAY
        self.IMESSAGE_DELAY = max([ru.forInterface(IInstantMessageSettings).imessage_delay*1000, MIN_IMESSAGE_DELAY])
        self.ESI_ROOT = self.portal_url
        
        return self.html_template()

    def get_senders_latest_message(self):
        """ Returns the latest message of all senders
        slm: senders_latest_message
        """
        t1 = time()
        slm = []
        
        context = aq_inner(self.context)
        mt      = getToolByName(context,'portal_membership')
        user_id = mt.getAuthenticatedMember().getId()
        
        if user_id:
            slm = db.getLatestMessageOfAllSenders(receiver_id=user_id)
            slm = self.mu.convertMessagesForTemplate(context, slm)
            print "\t\tDAUER get_senders_latest_message (ms):", (time()-t1) * 1000
            return slm
        else:
            return None
        
    def get_dialog(self,max=1000):
        """ Returns the latest message of a given sender
        slms: sender_latest_messages
        """
        t1 = time()
        slms = []
        
        context = aq_inner(self.context)
        sender_id = context.REQUEST.get('sid',False)
        
        mt      = getToolByName(context,'portal_membership')
        user_id = mt.getAuthenticatedMember().getId()
        
        if sender_id and user_id:
            slms = db.getDialog(receiver_id=user_id, 
                                 sender_id=sender_id, 
                                 order_field='timestamp',
                                 reverse_order=True, 
                                 max=max,offset=0, auth_user_id = user_id)              
            slms = self.mu.convertMessagesForTemplate(context, slms)
            print "\t\tDAUER get_dialog (ms):", (time()-t1) * 1000
            return slms
        else:
            return None

    def use_groups(self):
        """check if the auth user can create group chats
        """
        context =     aq_inner(self.context)       
        mt      =     getToolByName(context,'portal_membership')
        auth_user =   mt.getAuthenticatedMember()
        site_admins = api.user.get_users(groupname='Site Administrators')
        if auth_user in site_admins:
            return 1
        else:
            return 0
    
    def sender_data(self):
        """ gets sender data
        """
        context = aq_inner(self.context)
        mt = getToolByName(context, "portal_membership")
        sid = context.REQUEST.get('sid', None)
        member = mt.getMemberById(sid)
        if not member:
            return False
        else:
            return {
                    "id"        : sid,
                    "name"      : member.getProperty("fullname"),
                    "portrait"  : member.getPersonalPortrait(sid, size="pico").absolute_url()
                    } 

     
class AjaxInstantMessageView(BrowserView):
    """Ajax View of an Instant Message
    """

    def __call__(self):
        
        self.mu = MessageUtils()
        self.is_ityou_esi_theme_available = isProductAvailable("ityou.esi.theme")
        
        context = aq_inner(self.context)
        request = context.REQUEST
        
        action = request.get('action')
                        
        if action == 'post_message':
            return self.add_message()
            
        elif action == 'get_messages':
            return self.get_messages()
            
        elif action == 'get_dialog':
            return self.get_dialog()

        elif action == 'message_read_by_receiver':
            return self.set_message_state("approved")
        
        elif action == 'message_hidden_by_receiver':
            return self.set_message_state("hide_receiver")

        elif action == 'message_hidden_by_sender':
            return self.set_message_state("hide_sender")

        elif action == 'message_delete':
            return self.set_message_state("deleted")

        else:
            pass    

    def get_dialog(self):
        """ Returns the dialog between to users 
        """
        context = aq_inner(self.context)
        request = context.REQUEST
        mt      = getToolByName(context,"portal_membership")

        t1 = time()

        timestamp     = request.get('timestamp')
        sender_id     = request.get('sender_id')
        max           = int(request.get('max',"10"))
        offset        = int(request.get('offset',"0"))
        approved      = request.get('approved',False)      
        user_id = mt.getAuthenticatedMember().getId()
        messages= db.getDialog(receiver_id=user_id, 
                               sender_id=sender_id, 
                               timestamp=timestamp,
                               order_field="id",
                               reverse_order=True,  
                               max=max,offset=offset,
                               approved=approved,
                               auth_user_id = user_id)
        messages = self.mu.convertMessagesForTemplate(context, messages)

        #LM@MR: TEST
        #return mu.jsonResponse(context, messages[:-1]) # -1 because the latest message should not be returnd       
        print "\t\tDAUER get_dialog [AJAX] (ms):", (time()-t1) * 1000  
        return self.mu.jsonResponse(context, messages)

    def set_message_state(self, state):
        """The message will be set to delete or read
        """
        context = aq_inner(self.context)
        request = context.REQUEST

        t1 = time()        
        mt      = getToolByName(context,"portal_membership")
        mids     = json.loads(request.get('mids'))
        
        user_id = mt.getAuthenticatedMember().getId()
        try:
            db.setMessageState(receiver_id=user_id, mids=mids,state=state)
        except:
            logging.error('Could not set message state! %s' % state)
        print "\t\tDAUER set_message_state (ms):", (time()-t1) * 1000
        return None


    def get_messages(self):
        """ Returns a list of message from the Database
        #LM vorher: DAUER get_messagee (ms): 364.3883776620
        #LM nachher: DAUER get_messagee (ms): 12.6121044159
        """
        context = aq_inner(self.context)
        request = context.REQUEST
        mt      = getToolByName(context,"portal_membership")
        t1 = time()

        timestamp       = request.get('timestamp', None)

        if timestamp:
            #timestamp       = datetime.strptime(timestamp, "%Y-%m-%d %H:%M:%S.%f")
            timestamp       = datetime.strptime(timestamp, "%Y-%m-%dT%H:%M:%S.%fZ") #TODO #LM
        
        sender_id       = request.get('sender_id', False)
        receiver_id     = request.get('receiver_id', False)
        newer           = request.get('newer',False)
        approved        = request.get('approved',False)
        omit_sender     = request.get('omit_sender',False)    
        max             = int(request.get('max',"5"))
        offset          = int(request.get('offset',"0"))
        dialog          = request.get('dialog', False)
        reverse_order   = request.get('reverse_order', True)
        
        user_id = mt.getAuthenticatedMember().getId()

        tx = time()        
        if not receiver_id:
            receiver_ids = self._getReceiverIdsForUser(user_id)
        else: 
            receiver_ids = [receiver_id,]

        if self.is_ityou_esi_theme_available and not receiver_id:
            rdb.delStatus("imessage", uid = user_id)

        messages = db.getMessages(receiver_ids=receiver_ids, 
                                 sender_id=sender_id, 
                                 timestamp=timestamp, 
                                 newer=newer,
                                 order_field='timestamp',
                                 reverse_order= reverse_order, 
                                 max=max,offset=offset,
                                 approved=approved, 
                                 auth_user_id = user_id,
                                 omit_sender = omit_sender, 
                                 dialog = dialog)

        messages = self.mu.convertMessagesForTemplate(context, messages)

        print "\t\tDAUER get_messages (ms):", (time()-t1) * 1000
        return self.mu.jsonResponse(context, messages)


        
    def add_message(self):
        """ Add a message to the Database
        """
        context = aq_inner(self.context)
        request = context.REQUEST
        mt      = getToolByName(context,"portal_membership")

        t1 = time()

        receiver_id         = request.get('receiver_id', False)
        receiver_group_id   = request.get('receiver_group_id', False)
        receiver_group_hash = request.get('receiver_group_hash', False)
        message_text        = request.get('message')
        new_group           = request.get('new_group', False)
        
        # Zum hinzufügen von Nachrichten muss mindestens ein Text + ein Empfänger (Benutzer, Gruppe oder iMessage Gruppe) angegeben sein
        if not ((receiver_id or receiver_group_id or receiver_group_hash) and message_text):
            return None

        # Empfänger als Liste
        if receiver_id:
            receiver_ids = receiver_id.replace(" ","").split(',')
        else:
            receiver_ids = []
        
        # Plone Gruppen als Liste
        if receiver_group_id:
            receiver_group_ids = receiver_group_id.replace(" ","").split(',')
        else:
            receiver_group_ids = []

        # Überorüfung ob Benutzer existieren
        for receiver_id in receiver_ids:
            if receiver_id not in context.acl_users.getUserNames():
                return None
        
        # Überprüfung ob Gruppen existieren
        for receiver_group_id in receiver_group_ids:
            try:
                api.group.get(groupname = receiver_group_id)
            except:
                return None
            
        # Utils initalisieren
        #######mu = MessageUtils()
        
        # Benutzerdaten
        user            = mt.getAuthenticatedMember()
        user_id         = user.getId()
        user_name       = user.getProperty('fullname','').decode('utf-8')
        user_email      = user.getProperty('email', '').decode('utf-8')
        user_portrait   = self.mu.getPersonalPortrait(user_id, size='pico')

        # Wenn iMessage-Gruppe erstellt werden soll,
        # dann Empfänger (Plone-Gruppen und Plone-Benutzer) zu
        # einer Liste vereinen
        if new_group and not receiver_group_hash:
            receiver_ids.extend(receiver_group_ids)
            receiver_ids.append(user_id)
            # Gruppe in Datenbank erstellen und hash speichern
            receiver_group_hash = db.createGroup(receiver_ids, new_group)
            
        # Gruppen-Hash als receiver_id
        if receiver_group_hash:
            receiver_ids = [receiver_group_hash,]

        # Liste mit Benutzern, die benachrichtigt werden sollen (statusflags) initialisieren
        sf_ids = []

        for receiver_id in receiver_ids:
            try:
                receiver_email  = mt.getMemberById(receiver_id).getProperty('email', '').decode('utf-8')
            except:
                receiver_email  = ""
            if new_group:
                receiver_id = receiver_group_hash
            else:
                receiver_id     = receiver_id.decode('utf-8')
            
            for r in IMESSAGE_REGEX:
                regex = re.compile(r["search"])
                message_text = regex.sub(r["replace"], message_text)
                        
            message = {  
                'sender_id':      user_id,      
                'sender_name':    user_name,
                'sender_email':   user_email,
                'receiver_id':    receiver_id,
                'receiver_name':   "",
                'receiver_email': receiver_email,
                'message':        message_text.decode('utf-8'),                                
                }

            latest_message = db.getMessage(request.get('hash'))
            
            timeout = datetime.now() - timedelta(seconds=120)
            
            update = False
            
            message['message']  = message_text.replace('[LF]', '\n').replace('\n', '<br />')

            # Bei Gruppen kein updateMessage!
            nu = NotifyUtils()
            if latest_message and not receiver_group_hash:
                if (latest_message['sender_id'] == message['sender_id']) and (latest_message['timestamp'] > timeout):                
                    update = True
                    message_confirm = db.updateMessage(latest_message['md5'],message['message'], message['receiver_id'])
                else:
                    message_confirm = db.addMessage(message)
            else:
                message_confirm = db.addMessage(message)
    
            # nur notify wenn message in db
            if message_confirm:
                nu.notify(message, message_confirm, mt)

            message['hash']             = message_confirm['md5']
            message['sender_portrait']  = user_portrait
            message['timestamp']        = message_confirm['timestamp'].strftime('%Y-%m-%dT%H:%M:%S.%fZ')
            message['timestr']          = message_confirm['timestamp'].strftime(TIME_STRING)
            message['update']           = update

            # Wenn iMessage-Gruppe, dann Namen der Gruppe aus Datenbank lesen
            if receiver_group_hash:
                message['receiver_name'] = db.getGroupname(receiver_id)
            else:
                # ansonsten Benutzername (Empfänger) oder ID
                try:
                    message['receiver_name'] = mt.getMemberById(receiver_id).getProperty("fullname")
                except:
                    message['receiver_name'] = receiver_id

            # Statusflags setzen
            # =======================================================================
            if self.is_ityou_esi_theme_available:
                ##from ityou.esi.theme.dbapi import DBApi as StatusFlagsApi
                ##sfapi = StatusFlagsApi()

                if receiver_group_hash:
                    sf_ids = db.getReceiversOfGroup(receiver_group_hash)
                elif receiver_id in receiver_group_ids:                    
                    groupmembers = api.user.get_users(groupname=receiver_id)
                    for groupmember in groupmembers:
                        groupmember_id = groupmember.getId()
                        sf_ids.append(groupmember_id)
                else:
                    sf_ids = [receiver_id,]
                for user in sf_ids:
                    if user == user_id:
                        continue
                    #####sfapi.setStatus("imessage", True, uid = user)
                    rdb.setStatus("imessage", 1, uid = user)
                    #print "IMESSAGE", user, 1
            # =======================================================================
                
            # Wenn iMessage-Gruppe, nicht traversieren
            if receiver_group_hash:
                break

        ###mu = MessageUtils()
        print "\t\tDAUER add_message (ms):", (time()-t1) * 1000

        if who_rdb: 
            who_rdb.setOnlineUser(user_id)
            rdb.setStatus("whoisonline", 1,  uid = user_id, omit_uid = True)

        if receiver_ids:
            return self.mu.jsonResponse(context, message)


    def _getReceiverIdsForUser(self, user_id):
        """ Calculates receiver_ids for user (with groups)
        """
        t1 = time()
        receiver_ids = []
        receiver_ids.append(user_id)
        user_groups = api.group.get_groups(username=user_id)
        for group in user_groups:
            receiver_ids.append(group.getId())
        receiver_ids.extend(db.getGroupsOfUser(receiver_ids))
        print "\t\tDAUER _getReceiverIdsForUser (ms):", (time()-t1) * 1000
        return receiver_ids


class NotifyUtils():
    """Notify utilities
    """

    def notify(self, nm, message_confirm, mt):
        """ write notification to notity-Database
        """

        # pruefe ob notify message eingeschaltet ist
        if not getUtility(IRegistry).forInterface(INotifySettings).imessage_notifications_active:
            return False

        #LM In der Reciever-ID kann auch der Hash der Chat-Gruppe stehen
        uid = mt.getMemberById(nm['receiver_id'])
        auth = mt.getAuthenticatedMember().getId()

        dbapi_notify = NotifyDBApi()
        receivers = []
        chat_group_hash = None

        if uid:
            # dann ist es ein user
            is_notify_receiver = db_extuserprofile.getProperty(nm['receiver_id'], "imessage_notifies")
            if is_notify_receiver:
                receivers = [nm['receiver_id']]
        else:            
            # dann ein chat-gruppen hash
            chat_group_hash = nm['receiver_id']

            # alle empfänger finden
            receivers = db.getReceiversOfGroup(chat_group_hash)

        for receiver_id in receivers:

            if auth != str(receiver_id): # keine mails an sicht selbst verschicken
                if db_extuserprofile.getProperty(receiver_id, "imessage_notifies"):
                    receiver = mt.getMemberById(receiver_id)
                    if chat_group_hash:
                        link = "@@messages?rid=" + chat_group_hash
                    else:
                        link = "@@messages?id=" + receiver_id
                                        
                    dbapi_notify.addNotification({
                        "action":           u'imessage',
                        "sender_id":        nm['sender_id'],
                        "sender_name":      nm['sender_name'],
                        "sender_email":     nm['sender_email'],
                        "receiver_id":              receiver_id,
                        "receiver_name":            receiver.getProperty('fullname'),
                        "receiver_email":           receiver.getProperty('email'),
                        "content_uid":              None,
                        "content_path":             link,
                        "content_title":            None,
                        "message":          nm["message"],
                        "timestamp_mod":    message_confirm['timestamp'],
                         }) 

        return True


class MessageUtils():
    """ Small Utils to render Messages
    """

    def __init__(self):
        self.is_extuserprofile_available = isProductAvailable('ityou.extuserprofile')

    def convertMessagesForTemplate(self, context, ms):
        """ Converts the message dict to what the template needs
        #LM vorher: DAUER convertMessagesForTemplate (ms): 4587.637736637
        #LM nachher: DAUER convertMessagesForTemplate (ms): 63.873052597
        #LM Todo: Logik raus - hier geht es nur im die Aufbreitung der Daten für das Template
        """
        t1 = time()
        messages = []
        if not ms:
            return messages

        mt = getToolByName(context,'portal_membership')
        auth_member = mt.getAuthenticatedMember()
        auth_member_id = auth_member.getId()

        for m in ms:
            #if not m: #LM hää?
            #    continue

            sender_id =     m['sender_id']

            sender = mt.getMemberById(m['sender_id'])            
            if not sender:                
                #LM wenn der Sender nicht mehr existriert, muss die Nachricht raus 2014-05-23
                continue
             
            #LM 2014-05-23   
            #try:
            #    sender    =   mt.getMemberById(m['sender_id'])
            #    sender_exists = True
            #except:
            #    sender = False
            #    sender_exists = False
            #    #LM wenn der Sender nicht mehr existriert, muss die Nachricht raus 2014-05-23
            #    continue

            #LM 2014-05-17
            if sender:
                sender_name = sender.getProperty("fullname")
            else:
                sender_name = '' 
            receiver_id =   m['receiver_id']
            
            is_grouped = False
            if context.REQUEST.get('dialog', False):
                for message in messages:
                    if message["sender_id"] == sender_id and message["receiver_id"] == receiver_id \
                        or message["receiver_id"] == sender_id and message["sender_id"] == receiver_id \
                        or message["receiver_id"] == receiver_id and not mt.getMemberById(message["receiver_id"]):
                        if not m['approved'] and m['sender_id'] != auth_member_id:
                            message['unread'] +=1
                        is_grouped = True
                        break
                if is_grouped:
                    continue

            # if group, else receivers is None
            group_receivers =     db.getReceiversOfGroup(receiver_id)

            """ #LM raus
            group = False
            try:
                receiver_name     = mt.getMemberById(m['receiver_id']).getProperty("fullname")
                ##group = False
            except:
                if(len(receivers) > 0):
                    group = True
                receiver_name   = db.getGroupname(receiver_id)
            """
            # ist der Empfänger eine Gruppe oder ein einzelner Benutzer?

            if mt.getMemberById(m['receiver_id']):
                receiver_name = mt.getMemberById(m['receiver_id']).getProperty("fullname")
                group = False
            elif group_receivers: #LM: es kann vorkommen, dass es Gruppen ohne Benutzer gibt
                receiver_name = db.getGroupname(receiver_id)
                group = True
            else: #LM: kommt vor wenn es den receiver nicht mehr gibt, dann raus damit
                continue

            #print "Group / RECEIVER_NAMES / RECEIVERS: ", group, receiver_name, group_receivers

            # den tag() übertragen da mit json sonst nicht serialisierbar
            sender_portrait   = self.getPersonalPortrait(sender_id, size='pico')
            try:
                receiver_portrait = self.getPersonalPortrait(receiver_id, size='pico')
            except:
                receiver_portrait = "defaultUser.png"

            timestr         = m['timestamp'].strftime(TIME_STRING)
            
            #LM raus 'sender_exists':    sender_exists,
            m_tmpl = {
                'sender_id':        sender_id,
                'sender_name':      sender_name,
                'sender_email':     m['sender_email'],
                'sender_portrait':  sender_portrait,
                'receiver_id':      m['receiver_id'],
                'receiver_name':    receiver_name,
                'receiver_email':   m['receiver_email'],
                'receiver_portrait': receiver_portrait,
                'message':          m['message'],
                'timestamp':        m['timestamp'].strftime('%Y-%m-%dT%H:%M:%S.%fZ'),
                'timestr':          timestr,
                'hash':             m['md5'],
                'site_url':         getSite().absolute_url(),
                'approved':         m['approved'],
                'deleted':          m['deleted'],
                'group':            group,
                'unread':           0
            }
            if not m_tmpl['approved'] and m_tmpl['sender_id'] != auth_member_id:
                m_tmpl['unread'] += 1
                
            # Prüfen, ob dies eine Gruppe ist
            #### s.o. ### receivers = db.getReceiversOfGroup(receiver_id)
            receiver_list = []
            if group:
                for r in group_receivers:
                    try:
                        user = mt.getMemberById(r)
                        receiver_id = user.getId()
                        receiver_name = user.getProperty("fullname")
                        tx = time()
                        receiver_list.append({"id"  : receiver_id, "name"   : receiver_name, "portrait" : self.getPersonalPortrait(receiver_id, size="icon") })
                    except:
                        group = api.group.get(groupname = r)
                        if group:
                            group_members = group.getGroupMembers()
                            for member in group_members:
                                member_id = member.getId()
                                in_receivers = False
                                for tr in receiver_list:
                                    if tr["id"] == member_id:
                                        in_receivers = True
                                if not in_receivers and not sender_id == member_id:
                                    tx = time()
                                    receiver_list.append({"id"  : member_id, "name"   : member.getProperty("fullname"), "portrait" : self.getPersonalPortrait(member_id, size="icon") })
            m_tmpl['multiple_receivers'] = receiver_list
            messages.append(m_tmpl)

        print "\t\tDAUER convertMessagesForTemplate (ms):", (time()-t1) * 1000
        return messages

    def jsonResponse(self, context, data):
        """ Returns Json Data in Callback function
        """
        request = context.REQUEST
        callback = request.get('callback','')        
        request.response.setHeader("Content-type","application/json")
        if callback:
            cb = callback + "(%s);"
            return cb % json.dumps(data)
        else:
            return json.dumps(data)

##class PersonalPortraits():
##    """Get url off personal portraits
##    """

    @memoize    
    def getSmallPortrait(self,portrait,username,size="small-personal-portrait"):
        """ rermoves width and height attributes
            and adds a class to the img tag
            if jquery is rendering the image, we only need
            the src of the image
        """
        soup = bs(portrait)
        img = soup.img
        return str(img['src'])

    @memoize
    def getPersonalPortrait(self,id=None, verifyPermission=0, size=None):
        """Adapts the original getPersonalPortrait 
        If ityou.extpersonalportrait is installed, the
        patched personal portrai will be called, else the
        default
        #LM if obj ist returned:
            DAUER getPersonalPortrait [MESSAGE] (ms): 383.431196213
        #LM if url is returned:
            DAUER: >>>>>>>>>>>>>>  0.030 ms
        """
        t1 = time()
        plone = getSite()
        mt = getToolByName(plone,"portal_membership")

        if self.is_extuserprofile_available:
            return mt.getPersonalPortrait(id, size=size).absolute_url()
        else:
            return mt.getPersonalPortrait(id=id).absolute_url()


class AjaxUserView(BrowserView):
    """Get Plone-User with ajax
        ===> returns json 
    """
    
    def __call__(self):
        """Standard Json view
        """
        context = aq_inner(self.context)
        request = context.REQUEST
        action  = request.get('action')

        if action == "query":
            return self.query()
        else:
            return False

    def query(self):
        """Query Members
        """
        LIMIT     = 10000

        context   = aq_inner(self.context)
        request   = context.REQUEST

        q         = request.get('q','')
        limit     = int(request.get('l',LIMIT))
        mt        = getToolByName(context,'portal_membership')

        if q:
            users = mt.searchForMembers(name=q)
        else:
            users = mt.listMembers()

        ju = MessageUtils()

        return ju.jsonResponse( context,  self._convert_users(users[:limit])   )

    def _convert_users(self, users):
        """Converts Memberdata = Default user profile + extended!
        """
        context  = aq_inner(self.context)        
        user_list = []
        for u in users:
            #ToDo
            uid = u.getId().decode('utf-8')
            eup = db_extuserprofile.getExtendedProfile(uid)       
            user_list.append({
              "id":         u.getId(), 
              "name":       u.getProperty('fullname'),
              "email":      u.getProperty('email'),  
              "portrait":   u.getPersonalPortrait(u.getId(),size='pico'),
              "phone":      eup.phone or '',
              "location":   u.getProperty("location"),
              "position":   eup.position or '',
              "last_login": context.toLocalizedTime(u.getProperty("login_time"),long_format=1),
              "last_login_timestamp": int(u.getProperty("login_time"))
            })
        return user_list





# -*- coding: utf-8 -*-
import logging
from time import time
import json
import hashlib
import re
from datetime import datetime, date, timedelta

from Acquisition import aq_inner

from zope.component.hooks import getSite
from zope.component import getUtility

from plone import api

from plone.registry.interfaces import IRegistry
from plone.portlets.interfaces import IPortletDataProvider

from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName


# -------------------------------------
from ityou.imessage.dbapi import DBApi
db = DBApi()  


class TestInstantMessageView(BrowserView):
    """TEST View of an Instant Message
    """


    def __call__(self):
        """ TESTS
        """        
        context = aq_inner(self.context)

        messages = db.getMessagesForDialogOverview(\
                    receiver_ids =  ['lmeise', 'hmustermann'], \
                    sender_id =     'lmuller', \
                    timestamp =     datetime.now(), \
                    newer =         False, \
                    group_field =   'sender_id', \
                    order_field =   "id", \
                    reverse_order = True, \
                    approved =      False, \
                    omit_sender =   False, \
                    max=20, offset = 0, 
                    auth_user_id =  'lmuller')

        context.REQUEST.RESPONSE.setHeader('content-type', 'text')
        return "===============\n" + str(len(messages)) + '\n' + str(messages)
        

    










